/**
 * Logger for list.
 * @memberof module:apeman-task
 * @constructor ListLogger
 * @returns A logger instance.
 */

'use strict'

const colorprint = require('colorprint')
const util = require('util')
const path = require('path')
const textconv = require('textconv')
const arraysort = require('arraysort')
const colors = colorprint.colors

/** @lends ListLogger */
module.exports = colorprint.define(
  /** @lends ListLogger.prototype */
  {
    /**
     * Log task list.
     * @param {object} tasks - Tasks to show.
     * @param {object[]} childTasks - Child tasks to show.
     */
    logList (tasks, childTasks) {
      const NAME_MIN_LENGTH = 24
      const s = this
      s.debug('')
      s.debug('  Available tasks:')
      s.debug('')
      Object.keys(tasks)
        .sort(arraysort.stringCompare())
        .forEach((name) => {
          let desc = s._descTask(tasks[ name ])
          let comment = desc ? colors.blackBright([ '//', desc ].join(' ')) : ''
          s.debug(`    ${colors.black.bold(textconv.pad(name, NAME_MIN_LENGTH))}${comment}`)
        })
      s.debug('')
      childTasks = (childTasks || [])
      childTasks.forEach((data) => {
        let namespace = data.namespace
        let cwd = data.cwd
        cwd = (cwd && path.relative(process.cwd(), cwd)) || null
        let location = (cwd ? '"./' + cwd + '"' : '')
        s.debug(colors.black.italic('    /* Tasks in ' + (location || namespace) + ' */'))
        Object.keys(data.tasks)
          .sort(arraysort.stringCompare())
          .forEach((name) => {
            let desc = s._descTask(data.tasks[ name ])
            let fullName = [ namespace, name ].join('#')
            let comment = desc ? colors.blackBright([ '//', desc ].join(' ')) : ''
            s.debug(`    ${colors.black.bold(textconv.pad(fullName, NAME_MIN_LENGTH))}${comment}`)
          })
        s.debug('')
      })
      s.debug('')
      s.debug('')
    },
    _descTask (task) {
      const s = this
      if (Array.isArray(task)) {
        return task
          .map((name) =>
            s._descTask(name) || ('`' + String(name) + '`')
          )
          .map((desc) =>
            textconv.strip(desc.trim(), /\./)
          ).join(', ')
      }
      let $desc = task.$desc || task.$description
      if ($desc) {
        return $desc
      }
      if (typeof(task) === 'function') {
        return util.format('function() {/* ... */}')
      }
      return task.desc || ''
    }
  }
)
