/**
 * @function runTask
 * @memberof module:apeman-task
 * @param {Apemanfile} configuration - Apemanfile configuration.
 * @param {string|string[]} names - Names of tasks to run.
 * @param {object} options - Optional settings.
 * @param {number}  [options.depth=0] - Task tree depth.
 * @returns {Promise}
 */

'use strict'

const assert = require('assert')
const co = require('co')
const RunLogger = require('../logging/run_logger')
const ContextLogger = require('../logging/context_logger')
const apemanfile = require('apemanfile')
const argx = require('argx')

/** @lends runTask */
function runTask (configuration, names, options) {
  let args = argx(arguments)
  configuration = args.shift()
  options = args.pop('object') || {}
  names = [].concat(args.shift('string|function|array'))
    .filter((name) => !!name)
    .reduce((a, b) => [].concat(a, b), [])

  let depth = Number(options.depth || 0)
  let silent = !!options.silent
  let verbose = !!options.verbose

  assert.ok(configuration, 'configuration is required.')

  let logger = new RunLogger({
    indent: depth,
    verbose: verbose,
    disabled: silent
  })

  return co(function * () {
    for (let name of names) {
      if (/#/.test(name)) {
        yield runTask.childTask(configuration, name, options)
        break
      }
      let task = typeof name === 'function' ? name : configuration.get(`$tasks.${name}`)
      if (!task) {
        throw new Error(`Unknown task: ${name}`)
      }
      let startAt = new Date()
      if (options.namespace) {
        logger.logTaskStarted([ options.namespace, name ].join('#'), task)
      } else {
        logger.logTaskStarted(name, task)
      }
      for (let aTask of [].concat(task)) {
        switch (typeof aTask) {
          case 'string':
            yield runTask.nestedTask(configuration, aTask, options)
            break
          case 'function':
            yield new Promise(
              (resolve, reject) => {
                let promise = aTask({
                  apemanfile: configuration,
                  depth: depth,
                  verbose: verbose,
                  logger: new ContextLogger({
                    disabled: silent,
                    depth: depth + 1
                  })
                }, (err, result) => {
                  console.warn('[apeman-task] callback is now deprecated. Task plugin should return a promise now.')
                  err ? reject(err) : resolve(result)
                })
                if (promise) {
                  promise.then(resolve, reject)
                }
              }
            )
            break
          default:
            throw new Error('Task must be function or string, but given: ' + String(task))
        }
      }
      logger.logTaskFinished(Number(new Date() - startAt))
    }
  })
    .catch((err) => {
      logger.logTaskFailed(err)
      process.exit(1)
    })
}

runTask.nestedTask = function runSubTask (configuration, name, options) {
  let args = argx(arguments)
  configuration = args.shift()
  name = [].concat(args.shift('string|array'))
  options = args.pop('object') || {}

  let subOptions = Object.assign({}, options)
  subOptions.depth = Number(options.depth || 0) + 1
  let runTask = require('./run_task')
  return runTask(configuration, name, subOptions)
}

runTask.childTask = function (configuration, name, options) {
  let namepaths = name.split(/#/g)
  let namespace = namepaths.shift()
  if (options.namespace) {
    options.namespace = [ options.namespace, namespace ].join('#')
  } else {
    options.namespace = namespace
  }
  return co(function * () {
    let $child = configuration.$children[ namespace ]
    if (!$child) {
      throw new Error('$child not found with name: ' + name)
    }
    let here = process.cwd()
    let cwd = $child.$cwd || process.cwd()
    process.chdir(cwd)
    let result = yield runTask(apemanfile($child), namepaths.join('#'), options)
    process.chdir(here)
    return result
  })
}
module.exports = runTask
